--[[--------------
Do NOT edit this file if you don't know what you're doing.
Config options have been moved into the config.lua file.
]]----------------
local addon, ns = ...
local f = CreateFrame("Frame", "SacriRaidBuffed", UIParent)
--[[
	Buff List
	http://www.wowhead.com/guide=1100
]]--

local applyDragFunctionality = function(f,userplaced,locked)
	f:SetScript("OnDragStart", function(s) if IsShiftKeyDown() then s:StartMoving() end end)
	f:SetScript("OnDragStop", function(s) s:StopMovingOrSizing() end)

	local t = f:CreateTexture(nil,"OVERLAY",nil,6)
	t:SetAllPoints(f)
	t:SetTexture(0,1,0)
	t:SetAlpha(0)
	f.dragtexture = t    
	f:SetHitRectInsets(-50,-50, -50, -50)

	if not userplaced then
		f:SetMovable(false)
	else
		f:SetMovable(true)
		f:SetUserPlaced(true)
		if not locked then
			f.dragtexture:SetAlpha(0.2)
			f:EnableMouse(true)
			f:RegisterForDrag("LeftButton")
			f:SetScript("OnEnter", function(s) 
				GameTooltip:SetOwner(s, "ANCHOR_TOP")
				GameTooltip:AddLine(s:GetName(), 0, 1, 0.5, 1, 1, 1)
				GameTooltip:AddLine("Hold down ALT+SHIFT to drag!", 1, 1, 1, 1, 1, 1)
				GameTooltip:Show()
				end)
			f:SetScript("OnLeave", function(s) GameTooltip:Hide() end)
		else
			f.dragtexture:SetAlpha(0)
			f:EnableMouse(nil)
			f:RegisterForDrag(nil)
			f:SetScript("OnEnter", nil)
			f:SetScript("OnLeave", nil)
		end
	end  
end
applyDragFunctionality(f, ns.cfg.userplaced, true)
  
f.n = NUM_LE_RAID_BUFF_TYPES
if ns.cfg.showfoodFlask and ns.cfg.showHeroism then     --both heroism and food/flask
	f.n = NUM_LE_RAID_BUFF_TYPES+3
elseif ns.cfg.showfoodFlask and not ns.cfg.showHeroism then --only food/flask
	f.n = NUM_LE_RAID_BUFF_TYPES+2
elseif ns.cfg.showHeroism and not ns.cfg.showfoodFlask then --only heroism
    f.n = NUM_LE_RAID_BUFF_TYPES+1
end



if ns.cfg.vert then
	f:SetHeight((ns.cfg.iconsPerRowOrColumn * (ns.cfg.icondimension + ns.cfg.gap))-ns.cfg.gap)
	f:SetWidth((math.ceil(f.n / ns.cfg.iconsPerRowOrColumn) * (ns.cfg.icondimension + ns.cfg.gap))-ns.cfg.gap)
else
	f:SetWidth((ns.cfg.iconsPerRowOrColumn * (ns.cfg.icondimension + ns.cfg.gap))-ns.cfg.gap)
	f:SetHeight((math.ceil(f.n / ns.cfg.iconsPerRowOrColumn) * (ns.cfg.icondimension + ns.cfg.gap))-ns.cfg.gap)
end

f:SetPoint(ns.cfg.fp, ns.cfg.rel, ns.cfg.sp, ns.cfg.x, ns.cfg.y)


local function cap(str)
    return (str:gsub("^%l", string.upper))
end

do
	f.Locale = LibStub("LibBabble-TalentTree-3.0"):GetLookupTable()
	f.classes = {}
	local numClasses = GetNumClasses();
	local classDisplayName, classTag
	for i = 1, numClasses do
		classDisplayName, classTag = GetClassInfo(i);
		f.classes[classTag] = classDisplayName			
	end
end

local GetSpellInfo = GetSpellInfo
local UnitBuff = UnitBuff

function f:CreateButton(i)

	local btn = CreateFrame("Frame", "SRB"..i, f)
	btn.caster = nil
	
	local any = cap(SPELL_TARGET_TYPE1_DESC)
	local pet = cap(PET)
	
	local strany = "%s %s"				-- Any DRUID
	local strpet = "%s %s (%s: %s)"		-- Any WARLOCK (Pet: Imp)
	local bm = f.classes["HUNTER"]..": "..f.Locale["Beast Mastery"].." ("..pet..": ".."%s)"
	local str1spec = "%s: %s"			-- Shaman: Enhancement
	local str2spec = "%s: %s, %s"		-- DK: frost, unholy
	if i == 1 then
		btn.bt =  "Interface\\ICONS\\spell_magic_greaterblessingofkings"
		btn.desc = RAID_BUFF_1 --"Stats"
		btn.given = {
			strany:format(any, f.classes["PALADIN"]),
			strany:format(any, f.classes["DRUID"]),
			strany:format(any, f.classes["MONK"]),
			bm:format("Shale Spider"),
		}
	elseif i == 2 then
		btn.bt = "Interface\\ICONS\\spell_holy_wordfortitude"
		btn.desc = RAID_BUFF_2 --"Stamina"
		btn.given = {
			strany:format(any, f.classes["PRIEST"]),
			strany:format(any, f.classes["WARLOCK"]),
			strany:format(any, f.classes["WARRIOR"]),
			bm:format("Silithid"),
		}
	elseif i == 3 then
		btn.bt = "Interface\\ICONS\\ability_warrior_battleshout"
		btn.desc = RAID_BUFF_3 --"Attack Power"
		btn.given = {
			strany:format(any, f.classes["DEATHKNIGHT"]),
			strany:format(any, f.classes["HUNTER"]),
			strany:format(any, f.classes["WARRIOR"]),
			
		}
	elseif i == 4 then
		btn.bt = "Interface\\ICONS\\ability_rogue_disembowel"
		btn.desc = RAID_BUFF_4 --"Attack Speed"
		btn.given = {
			str2spec:format(f.classes["DEATHKNIGHT"], f.Locale["Frost"], f.Locale["Unholy"]),
			strany:format(any, f.classes["ROGUE"]),
			str1spec:format(f.classes["SHAMAN"], f.Locale["Enhancement"]),
			strpet:format(any, f.classes["HUNTER"], pet, "Hyena, Serpent"),
		}		
	elseif i == 5 then
		btn.bt = "Interface\\ICONS\\spell_fire_totemofwrath"
		btn.desc = RAID_BUFF_5 --"Spell Power"
		btn.given = {
			strany:format(any, f.classes["MAGE"]),
			strany:format(any, f.classes["SHAMAN"]),
			strany:format(any, f.classes["WARLOCK"]),
			bm:format("Waterstrider"),
		}
	elseif i == 6 then
		btn.bt = "Interface\\ICONS\\spell_nature_moonglow"
		btn.desc = RAID_BUFF_6--"Spell Haste"
		btn.given = {
			str1spec:format(f.classes["DRUID"], f.Locale["Balance"]),
			str1spec:format(f.classes["PRIEST"], f.Locale["Shadow"]),
			str1spec:format(f.classes["SHAMAN"], f.Locale["Elemental"]),
			strpet:format(any, f.classes["HUNTER"], pet, "Sporebat"),
		}
	elseif i == 7 then
		btn.bt = "Interface\\ICONS\\spell_nature_unyeildingstamina"
		btn.desc = RAID_BUFF_7--"Critical Strike"
		btn.given = {
			str2spec:format(f.classes["DRUID"], f.Locale["Guardian"], f.Locale["Feral"]),
			str1spec:format(f.classes["MONK"], f.Locale["Windwalker"]),
			strany:format(any, f.classes["MAGE"]),
			strpet:format(any, f.classes["HUNTER"], pet, "Hydra, Wolf"),
			bm:format("Devilsaur, Quilen, Waterstrider"),
		}
	elseif i == 8 then
		btn.bt = "Interface\\ICONS\\spell_holy_greaterblessingofkings"
		btn.desc = RAID_BUFF_8--"Mastery"
		btn.given = {
			strany:format(any, f.classes["PALADIN"]),
			strany:format(any, f.classes["SHAMAN"]),
			strpet:format(any, f.classes["HUNTER"], pet, "Cat"),
			bm:format("Spirit Beast"),
		}
	elseif i == 9 then
	    if ns.cfg.showfoodFlask then
	   
		btn.bt = "Interface\\ICONS\\spell_misc_food"
		btn.desc = (GetSpellInfo(104267))
		btn.given = {
			POWER_TYPE_FOOD
		}
		btn.caster = "player"
		else
		  if ns.cfg.showHordeMode then
		  btn.bt = "Interface\\ICONS\\spell_nature_bloodlust"
		  else
		  btn.bt = "Interface\\ICONS\\ability_shaman_heroism"
		  end
		btn.desc = ITEM_MOD_HASTE_RATING_SHORT --"Haste"
		btn.given = {
			strany:format(any, f.classes["MAGE"]),
			strany:format(any, f.classes["SHAMAN"]),
			bm:format("Core Hound"),
		}
		btn.alert = CreateFrame("Frame", "SRBOverlay", btn, "ActionBarButtonSpellActivationAlert");
		btn.alert:SetAllPoints(btn)
		end
	elseif i == 10 then
		btn.bt = "Interface\\ICONS\\trade_alchemy_potione5"
		btn.desc = (select(4, GetAuctionItemSubClasses(4))) --"Flask"
		btn.given = {
			(select(4, GetAuctionItemSubClasses(4))),
		}
		btn.caster = "player"
	elseif i == 11 then
		  if ns.cfg.showHordeMode then
		  btn.bt = "Interface\\ICONS\\spell_nature_bloodlust"
		  else
		  btn.bt = "Interface\\ICONS\\ability_shaman_heroism"
		  end
		btn.desc = ITEM_MOD_HASTE_RATING_SHORT --"Haste"
		btn.given = {
			strany:format(any, f.classes["MAGE"]),
			strany:format(any, f.classes["SHAMAN"]),
			bm:format("Core Hound"),
		}
		btn.alert = CreateFrame("Frame", "SRBOverlay", btn, "ActionBarButtonSpellActivationAlert");
		btn.alert:SetAllPoints(btn)
	end
		
	if ns.cfg.showTooltip then
	
		btn:SetScript("OnEnter", function(self) 
				GameTooltip:SetOwner(self, "ANCHOR_BOTTOMLEFT")
				GameTooltip:ClearLines()
				GameTooltip:AddLine(self.desc)
				for i = 1, #btn.given do
					GameTooltip:AddLine(btn.given[i])
				end
				if self.caster ~= nil then
					local _, classFileName = UnitClass(self.caster)
					local color = RAID_CLASS_COLORS[classFileName]
					GameTooltip:AddLine("Caster: "..GetUnitName(self.caster, false), color.r, color.g, color.b)
				end
				GameTooltip:Show()
			end)
			
		btn:SetScript("OnLeave", function(self) 
				GameTooltip:Hide()
			end)
	
	end
	
	btn:SetHeight(ns.cfg.icondimension)
	btn:SetWidth(ns.cfg.icondimension)
	btn.bg = btn:CreateTexture(nil, "BACKGROUND")
	btn.bg:SetAllPoints(btn)
	btn.bg:SetTexture(btn.bt)
	
	if not ns.cfg.usingcustomicons then
		btn.bg:SetTexCoord(.08, .92, .08, .92)
	end
	
	

	if ns.cfg.useaurora and IsAddOnLoaded("Aurora") then

		local auroraF, auroraC = unpack(Aurora)
		auroraF.SetBD(btn, 0,0,0,0)--, -1, 1, 1, -1)
		
	elseif ns.cfg.usebeautycase and IsAddOnLoaded("!Beautycase") then
	
		CreateBorder(btn, ns.cfg.bcbordersize, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1)
		
	elseif ns.cfg.addborder then	-- TY nin
	
		btn:SetBackdrop(ns.cfg.backdrop)
		btn:SetBackdropColor(0,0,0,0)
		btn:SetBackdropBorderColor(0,0,0)
		
	end

	
	btn:SetAlpha(ns.cfg.koA)
	btn:Show()
	
	
	--if ns.cfg.desaturate then
		--ns.cfg.okA = 1
		--ns.cfg.koA = 0
		--btn:SetAlpha(ns.cfg.okA)
		--btn.SetAlpha = function(btn, n)
		--	if n == 1 then 
		--		n = false 
		--	else 
		--		n = true
		--		btn.bg:SetTexture(btn.bt)
		--	end
		--	btn.bg:SetDesaturated(1)
		--end
	if not ns.cfg.desaturate then
		hooksecurefunc(btn, "SetAlpha", function(btn, i) 
			if i == ns.cfg.koA then
				btn.bg:SetTexture(btn.bt)
			end
		end)
	end
	
	
	return btn
end	

function f:GROUP_ROSTER_UPDATE()
	if GetNumGroupMembers() == 0 then
		f:Hide()
	else
		f:Show()
	end
end

function f:ADDON_LOADED(event, arg1, ...)

	if arg1 == "SacriRaidBuffed" then
		
		f:UnregisterEvent("ADDON_LOADED");
		for i=1,f.n do
	
			f[i] = f:CreateButton(i)

			ns.cfg.iconsPerRowOrColumn = math.floor(ns.cfg.iconsPerRowOrColumn)
			if ns.cfg.iconsPerRowOrColumn < 0 then
				ns.cfg.iconsPerRowOrColumn = f.n
			end
			
			if i == 1 then
				if ns.cfg.godown and ns.cfg.goleft then
					f[i]:SetPoint("TOPRIGHT", f, "TOPRIGHT", 0, 0)
				elseif ns.cfg.godown and ns.cfg.goleft == false then
					f[i]:SetPoint("TOPLEFT", f, "TOPLEFT", 0, 0)
				elseif not ns.cfg.godown and ns.cfg.goleft then
					f[i]:SetPoint("BOTTOMRIGHT", f, "BOTTOMRIGHT", 0, 0)
				else
					f[i]:SetPoint("BOTTOMLEFT", f, "BOTTOMLEFT", 0, 0)
				end
			else
				if ns.cfg.vert then
					if i % ns.cfg.iconsPerRowOrColumn == 1 then
						if ns.cfg.goleft then
							f[i]:SetPoint("RIGHT", f[i-ns.cfg.iconsPerRowOrColumn], "LEFT", -ns.cfg.gap, 0)
						else
							f[i]:SetPoint("LEFT", f[i-ns.cfg.iconsPerRowOrColumn], "RIGHT", ns.cfg.gap, 0)
						end
					else
						if ns.cfg.godown then
							f[i]:SetPoint("TOP", f[i-1], "BOTTOM", 0, -ns.cfg.gap)
						else
							f[i]:SetPoint("BOTTOM", f[i-1], "TOP", 0, ns.cfg.gap)
						end
					end
				else
					if i % ns.cfg.iconsPerRowOrColumn == 1 then
						if ns.cfg.godown then
							f[i]:SetPoint("TOP", f[i-ns.cfg.iconsPerRowOrColumn], "BOTTOM", 0, -ns.cfg.gap)
						else
							f[i]:SetPoint("BOTTOM", f[i-ns.cfg.iconsPerRowOrColumn], "TOP", 0, ns.cfg.gap)
						end
					else
						if ns.cfg.goleft then
							f[i]:SetPoint("RIGHT", f[i-1], "LEFT", -ns.cfg.gap, 0)
						else
							f[i]:SetPoint("LEFT", f[i-1], "RIGHT", ns.cfg.gap, 0)
						end
					end
				end				
			end
			
		end

		f:Show()
		if ns.cfg.showifalone == false then
			f:GROUP_ROSTER_UPDATE()
		end

	end

end


function f:CheckFlask()

	local _, texture
	
	local flasks = {105693, 105689, 105691, 105694, 105696} --, 79472 }
	for i = 1, #flasks do
		local name =(GetSpellInfo(flasks[i]))
		_, _, texture = UnitBuff("Player", name)
		if texture then
			return texture
		end
	end
	return nil
	
end

function f:CheckHero()
	 
	local _, texture
	local bursthaste =	{ 80353, 2825, 32182, 90355}--, 774} -- Rejuvenation (testing)
	
	for i = 1, #bursthaste do
		local name =(GetSpellInfo(bursthaste[i]))
		_, _, texture, _, _, _, _, caster = UnitBuff("Player", name)
		if texture then
			ActionButton_ShowOverlayGlow(f[f.n].alert)
			return texture, caster
		end
	end
	ActionButton_HideOverlayGlow(f[f.n].alert)
	return nil
end

-- Blizzard's GetRaidBuffTrayAuraInfo is bugged so..
f.buffs = {
	stat 		=	{ 90363, 20217,	115921, 1126 },
	stamina		= 	{ 469, 109773, 90364, 21562},
	atkpwr		= 	{ 19506, 57330,	6673 },
	atkspeed	=	{ 55610, 113742, 30809, 128432, 128433 },
	spllpwr		=	{ 77747, 109773, 126309, 61316, 1459 },
	spllhaste	= 	{ 24907, 15473, 51470, 49868, 135678 },
	crit		=	{ 17007, 1459, 61316, 116781, 97229, 24604, 90309, 126373, 126309 },
	mastery		=	{ 116956, 19740, 93435, 128997 },
}

local function GetRaidBuffTrayAuraInfo(i)
	if i == 1 then
		for x = 1, #f.buffs.stat do
			local name, _, texture, _, _, _, _, caster = UnitBuff("Player", (GetSpellInfo(f.buffs.stat[x])))
			if texture then
				return name, _, texture, caster
			end
		end
	elseif i == 2 then
		for x = 1, #f.buffs.stamina do
			local name, _, texture, _, _, _, _, caster = UnitBuff("Player", (GetSpellInfo(f.buffs.stamina[x])))
			if texture then
				return name, _, texture, caster
			end
		end
	elseif i == 3 then
		for x = 1, #f.buffs.atkpwr do
			local name, _, texture, _, _, _, _, caster = UnitBuff("Player", (GetSpellInfo(f.buffs.atkpwr[x])))
			if texture then
				return name, _, texture, caster
			end
		end
	elseif i == 4 then
		for x = 1, #f.buffs.atkspeed do
			local name, _, texture, _, _, _, _, caster = UnitBuff("Player", (GetSpellInfo(f.buffs.atkspeed[x])))
			if texture then
				return name, _, texture, caster
			end
		end
	elseif i == 5 then
		for x = 1, #f.buffs.spllpwr do
			local name, _, texture, _, _, _, _, caster = UnitBuff("Player", (GetSpellInfo(f.buffs.spllpwr[x])))
			if texture then
				return name, _, texture, caster
			end
		end
	elseif i == 6 then
		for x = 1, #f.buffs.spllhaste do
			local name, _, texture, _, _, _, _, caster = UnitBuff("Player", (GetSpellInfo(f.buffs.spllhaste[x])))
			if texture then
				return name, _, texture, caster
			end
		end
	elseif i == 7 then
		for x = 1, #f.buffs.crit do
			local name, _, texture, _, _, _, _, caster = UnitBuff("Player", (GetSpellInfo(f.buffs.crit[x])))
			if texture then
				return name, _, texture, caster
			end
		end
	elseif i == 8 then
		for x = 1, #f.buffs.mastery do
			local name, _, texture, _, _, _, _, caster = UnitBuff("Player", (GetSpellInfo(f.buffs.mastery[x])))
			if texture then
				return name, _, texture, caster
			end
		end
	else 
		return nil, nil, nil
	end

end

function f:PLAYER_AURAS_CHANGED()
    local sat = ns.cfg.desaturate
	for i=1,NUM_LE_RAID_BUFF_TYPES do
		local name, _, texture, caster = GetRaidBuffTrayAuraInfo(i);
		
		if name then 	
			if caster then
				f[i].caster = caster
			end
			f[i].bg:SetTexture(texture)
			f[i]:SetAlpha(ns.cfg.okA)
			if sat then
			f[i].bg:SetDesaturated(0)
			end
		else
			f[i].caster = nil
			f[i]:SetAlpha(ns.cfg.koA)
			if sat then
			f[i].bg:SetDesaturated(1)
			end
		end
	end
	
	if ns.cfg.showfoodFlask then
		local _, _, texture = UnitBuff("Player", (GetSpellInfo(104267)))
		if texture then
			f[NUM_LE_RAID_BUFF_TYPES+1].bg:SetTexture(texture)
			f[NUM_LE_RAID_BUFF_TYPES+1]:SetAlpha(ns.cfg.okA)
			
			if sat then
			f[NUM_LE_RAID_BUFF_TYPES+1].bg:SetDesaturated(0)
			end
		else
			f[NUM_LE_RAID_BUFF_TYPES+1]:SetAlpha(ns.cfg.koA)
			
			if sat then
			f[NUM_LE_RAID_BUFF_TYPES+1].bg:SetDesaturated(1)
			end
		end
		
		local flasked = f:CheckFlask()
		if flasked then
			f[NUM_LE_RAID_BUFF_TYPES+2].bg:SetTexture(flasked)
			f[NUM_LE_RAID_BUFF_TYPES+2]:SetAlpha(ns.cfg.okA)
			
			if sat then
			f[NUM_LE_RAID_BUFF_TYPES+2].bg:SetDesaturated(0)
			end
		else
			f[NUM_LE_RAID_BUFF_TYPES+2]:SetAlpha(ns.cfg.koA)
			
			if sat then
			f[NUM_LE_RAID_BUFF_TYPES+2].bg:SetDesaturated(1)
			end
		end
		

	end
	
	if ns.cfg.showHeroism then 
	
	   local hero, caster = f:CheckHero()
		if hero then
			if caster then
				f[f.n].caster = caster
			end
			f[f.n].bg:SetTexture(hero)
			f[f.n]:SetAlpha(ns.cfg.okA)
			
			if sat then
			f[f.n].bg:SetDesaturated(0)
			end
		else
			f[f.n].caster = nil
			f[f.n]:SetAlpha(ns.cfg.koA)
			
			if sat then
			f[f.n].bg:SetDesaturated(1)
			end
		end
	end
	
	
end

function f:PLAYER_ENTERING_WORLD()	
	hooksecurefunc(ConsolidatedBuffs, "Show", function() 
			ConsolidatedBuffs:Hide()
		end)
	ConsolidatedBuffs:UnregisterAllEvents()
	f:PLAYER_AURAS_CHANGED()
end

function f:UNIT_AURA(event, ...)
	f:PLAYER_AURAS_CHANGED()
end

function f:PLAYER_UNGHOST()
	f:PLAYER_AURAS_CHANGED()
end

function f:PLAYER_REGEN_ENABLED()
   f:SetAlpha(1)
end

function f:PLAYER_REGEN_DISABLED()
   f:SetAlpha(ns.cfg.fadeCombatAlpha)
end

function f:unlockFrame()
	print("Frames unlocked")
	print("/srb lock to lock")
	
	if f and f:IsUserPlaced() then
		if ns.cfg.showifalone == false then
			if not f:IsVisible() then
				f:Show()
			end
		end

		f.dragtexture:SetAlpha(0.2)
		f:EnableMouse(true)
		f:RegisterForDrag("LeftButton")
		
		f:SetScript("OnEnter", function(s) 
		GameTooltip:SetOwner(s, "ANCHOR_TOP")
		GameTooltip:AddLine(s:GetName(), 0, 1, 0.5, 1, 1, 1)
		GameTooltip:AddLine("Hold down SHIFT to drag!", 1, 1, 1, 1, 1, 1)
		GameTooltip:Show()
		end)
		f:SetScript("OnLeave", function(s) GameTooltip:Hide() end)
	end
end  
  
function f:lockFrame()
	print("Frames locked")
	print("/srb unlock to unlock")
	if f and f:IsUserPlaced() then
		if ns.cfg.showifalone == false then
			if f:IsVisible() then
				f:Hide()
			end
		end
		f.dragtexture:SetAlpha(0)
		f:EnableMouse(nil)
		f:RegisterForDrag(nil)
		f:SetScript("OnEnter", nil)
		f:SetScript("OnLeave", nil)
	end
end

function SacriRaidBuffed_slashHandler(arg1)

	print("SacriRaidBuffed is running")
	print("Options")
	print("/srb unlock")
	print("/srb lock")
	if arg1 == "unlock" then
		f:unlockFrame()
	elseif arg1 == "lock" then
		f:lockFrame()
	end
	
end


SlashCmdList["SacriRaidBuffed"] = SacriRaidBuffed_slashHandler
SLASH_SacriRaidBuffed1 = "/srb"

f:RegisterUnitEvent("ADDON_LOADED", "Player");
f:RegisterUnitEvent("UNIT_AURA", "Player");
f:RegisterUnitEvent("PLAYER_ENTERING_WORLD", "Player");
f:RegisterUnitEvent("PLAYER_AURAS_CHANGED", "Player");
f:RegisterUnitEvent("PLAYER_UNGHOST", "Player");
--f:RegisterUnitEvent("CVAR_UPDATE");

if ns.cfg.showifalone == false then
	f:RegisterEvent("GROUP_ROSTER_UPDATE")
end

if ns.cfg.fadeCombat then
    f:RegisterEvent("PLAYER_REGEN_ENABLED")
	f:RegisterEvent("PLAYER_REGEN_DISABLED")
end
f:SetScript('OnEvent', function(self, event, ...) self[event](self, event, ...) end)